@@ -15,7 +15,7 @@ import java.io.FileWriter;
 public class ControllerGUI extends JFrame implements ActionListener, KeyListener {
 	JList menu;
 	JScrollPane menuScroll;
-	JLabel title;
+	JComboBox title;
 	JLabel status;
 	JList scripts;
 	ControllerListener comm;
@@ -81,7 +81,10 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 				pre.append(" ");
 			JLabel title=new JLabel(pre.toString()+index+". "+me.title);
 			title.setBackground(me.selected?list.getSelectionBackground():list.getBackground());
-			title.setForeground(me.selected?list.getSelectionForeground():list.getForeground());
+			if(me.title.charAt(0)=='#')
+				title.setForeground(new Color(2f/3,0f,0f));
+			else
+				title.setForeground(me.selected?list.getSelectionForeground():list.getForeground());
 			title.setEnabled(list.isEnabled());
 			title.setFont(list.getFont());
 			title.setOpaque(true);
@@ -168,11 +171,13 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		Point p=menuScroll.getViewport().getViewPosition();  
 
 		//scripts.setEnabled(comm._isConnected);
+		title.setEnabled(comm._isConnected);
 		backBut.setEnabled(comm._isConnected);
 		refreshBut.setEnabled(comm._isConnected);
 		inputField.setEnabled(comm._isConnected);
+		title.removeAllItems();//clearing the title popup menu, but we'll refresh the items if we're still connected
 		if(!comm._isConnected) {
-			title.setText("-");
+			title.addItem("-");
 			menu.setListData(new Vector());
 			status.setText("Reconnecting...");
 		} else {
@@ -185,6 +190,8 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 					Iterator mons=comm.dynObjSrcs.values().iterator();
 					while(mons.hasNext())
 						comm.sendInput("!root "+(String)mons.next());
+					//Set focus to the main menu
+					menu.requestFocus();
 				}
 				for(int i=0; i<scriptsModel.getSize(); i++)
 					if(scriptsModel.get(i).toString().equals("CONNECT"))
@@ -192,7 +199,15 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 				comm._connectCount=-1; //so we don't get it again
 			}
 			synchronized(comm._menus) {
-				title.setText((String)comm._titles.lastElement());
+				int len=0;
+				for(Iterator it=comm._titles.iterator(); it.hasNext();) {
+					String entry=(String)it.next();
+					title.insertItemAt(entry,0);
+					if(entry.length()>len)
+						len=entry.length();
+				}
+				title.setMaximumSize(new Dimension((int)(len*8)+25,title.getHeight()));
+				title.setSelectedIndex(0);
 				Vector menuitems=(Vector)comm._menus.lastElement();
 				menu.setValueIsAdjusting(true);
 				menu.setListData((Vector)menuitems.clone());
@@ -226,6 +241,11 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		} else if(evt.getSource()==refreshBut) {
 			//System.out.println("refresh button clicked");
 			comm.sendRefresh();
+		} else if(evt.getSource()==title) {
+			//System.out.println("title selection changed");
+			int selected=title.getSelectedIndex();
+			for(int i=0; i<selected; i++)
+				comm.sendReturn();
 		} else if(evt.getSource()==inputField) {
 			//System.out.println("input: "+inputField.getText());
 			int[] sel=menu.getSelectedIndices();
@@ -260,11 +280,11 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 			estop.open();
 		} else if(evt.getActionCommand().equals("raw") || evt.getActionCommand().equals("rle")) {
 			if(evt.getActionCommand().equals("raw")) {
-				comm.sendInput("!root \"TekkotsuMon\" \"RawCamServer\"");
-				comm.dynObjSrcs.put("RawVisionGUI","\"TekkotsuMon\" \"RawCamServer\"");
+				comm.sendInput("!root \"TekkotsuMon\" \"Raw Cam Server\"");
+				comm.dynObjSrcs.put("RawVisionGUI","\"TekkotsuMon\" \"Raw Cam Server\"");
 			} else {
-				comm.sendInput("!root \"TekkotsuMon\" \"SegCamServer\"");
-				comm.dynObjSrcs.put("SegVisionGUI","\"TekkotsuMon\" \"SegCamServer\"");
+				comm.sendInput("!root \"TekkotsuMon\" \"Seg Cam Server\"");
+				comm.dynObjSrcs.put("SegVisionGUI","\"TekkotsuMon\" \"Seg Cam Server\"");
 			}
 		} else if(evt.getActionCommand().equals("addbookmark")) {
 			int sel=comm.firstSelected();
@@ -345,9 +365,10 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		
 		JPanel content=new JPanel(new BorderLayout());
 		Box titleBox=Box.createVerticalBox();
-		title=new JLabel("-");
-		title.setFont(new Font(title.getFont().getFontName(),Font.BOLD,title.getFont().getSize()));
+		title=new JComboBox();
+		title.addItem("-");
 		title.setAlignmentX(0);
+		title.addActionListener(this);
 		titleBox.add(title);
 		titleBox.add(Box.createVerticalStrut(spacer_size));
 		content.add(titleBox,BorderLayout.NORTH);
@@ -502,7 +523,8 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 			scriptsModel.addElement(new ScriptEntry(prefs.get("ControllerGUI.script"+i+".title","unknown"),prefs.get("ControllerGUI.script"+i+".cmd","")));
 
 		pack();
-		show();
+		title.setMaximumSize(new Dimension(15+25,title.getHeight()));
+		setVisible(true);
 		getRootPane().setMinimumSize(getRootPane().getSize());
 	}
 
@@ -530,34 +552,46 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		}
 	}
 
-	public void show() {
-		super.show();
-		menu.requestFocus();
+	public void setVisible(boolean b) {
+		super.setVisible(b);
+		if(b)
+			menu.requestFocus();
 	}
 	
 	public static void main(String s[]) throws java.lang.InterruptedException {
-    if (s.length>1) {
-      System.out.println("usage: java ControllerGUI [host]");
-      System.exit(1);
+    if (s.length>2 || s.length>0 && (s[0].equals("-h") || s[0].equals("--help"))) {
+      System.out.println("Usage: ControllerGUI [host [port]]");
+      System.out.println("       If host is unspecified, it will check previously used addresses");
+			System.out.println("       If port is unspecified, it will default to "+ControllerListener.defPort);
+			//			System.out.println("       If port is unspecified, it will be retrieved by ftp'ing the");
+			//			System.out.println("         current config file from host, and looking up the Controller");
+			//			System.out.println("         gui_port setting. (Default is "+ControllerListener.defPort+")");
+      System.exit(2);
     }
 
-    try {
-      DogConfig dogConfig=new DogConfig (s);
-      ControllerGUI controllerGUI=new ControllerGUI(dogConfig.getIP(),
-          Integer.parseInt(dogConfig.getValue("Controller", "gui_port")));
-
-      controllerGUI.addWindowListener(new WindowAdapter() {
-          public void windowClosing(WindowEvent e) { System.exit(0);} });
-			try{
-				DogConfigFTP dog_ftp=new DogConfigFTP(dogConfig.getIP(), 21, "config", "config");
-				String col=dogConfig.getValue("Vision","colors");
-				col=col.substring(col.lastIndexOf('/')+1);
-				PrintWriter out=new PrintWriter(new FileWriter("default.col"));
-				String tmp=dog_ftp.getFile(col);
-				out.print(tmp);
-				out.close();
-			} catch(Exception ex) { ex.printStackTrace(); }
-    } catch (IllegalArgumentException ex) {System.exit(0);}
+		ControllerGUI controllerGUI=null;
+		if(s.length==0) {
+			try {
+				DogConfig dogConfig=new DogConfig (s);
+				controllerGUI=new ControllerGUI(dogConfig.getIP(),Integer.parseInt(dogConfig.getValue("Controller", "gui_port")));
+			} catch (IllegalArgumentException ex) {System.exit(0);}
+		} else if(s.length>1)
+			controllerGUI=new ControllerGUI(s[0],Integer.parseInt(s[1]));
+		else
+			controllerGUI=new ControllerGUI(s[0],ControllerListener.defPort);
+		
+		controllerGUI.addWindowListener(new WindowAdapter() {
+				public void windowClosing(WindowEvent e) { System.exit(0);}
+			});
+		/*			try{
+						DogConfigFTP dog_ftp=new DogConfigFTP(dogConfig.getIP(), 21, "config", "config");
+						String col=dogConfig.getValue("Vision","colors");
+						col=col.substring(col.lastIndexOf('/')+1);
+						PrintWriter out=new PrintWriter(new FileWriter("default.col"));
+						String tmp=dog_ftp.getFile(col);
+						out.print(tmp);
+						out.close();
+						} catch(Exception ex) { ex.printStackTrace(); }*/
 	}
 
   public void keyPressed(KeyEvent e) {
