@@ -6,7 +6,7 @@ public class TestGraph {
     JFrame frame=new JFrame("graph test");
     GraphCanvas canvas=new GraphCanvas(0.0f, 1.0f);
     frame.setSize(500,300);
-    frame.show();
+    frame.setVisible(true);
     frame.getContentPane().add(canvas);
     while (true) {
       canvas.registerValue((float)Math.random());
